/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2012  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 3.86k                                            *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : RTOSInit_V850E2S_FG4.c
          for RENESAS V850E2S_Fx4 CPUs

Purpose : Initializes and handles the hardware for the OS as far
          as required by the OS.
          Feel free to modify this file acc. to your
          target system.

-------- END-OF-HEADER -----------------------------------------------
*/

#include "RTOS.h"

#ifndef __ICCV850__
  #include "df3580_0.h"
#else
  #include "io70f3580.h"
  #include <intrinsics.h>
#endif

/*********************************************************************
*
*       Configuration
*
**********************************************************************
*/

/*********************************************************************
*
*       Clock frequency settings
*/
#ifndef   OS_FSYS                /* Default is 64 MHz,                   */
  #define OS_FSYS 64000000uL     /* depends on crystal and PLL           */
#endif                           /* Some CPUs may be limited to 48 MHz ! */

#ifndef   OS_PCLK_TIMER
  #define OS_PCLK_TIMER  (OS_FSYS / 8uL)   /* Prescaler for OSTimer PCLK */
#endif

#ifndef   OS_INIT_PLL            /* May be ddefined as project option    */
  #define OS_INIT_PLL (1)        /* when simulator is used               */
#endif

/*********************************************************************
*
*       Configuration of communication to OSView
*/
#ifndef   OS_VIEW_ENABLE            // Global enable of communication
  #define OS_VIEW_ENABLE    (1)     // Default: on
#endif

#ifndef   OS_VIEW_USE_UART          // If set, UART will be used
  #define OS_VIEW_USE_UART  (1)     // Default: 1 => Uart is used
#endif                              // if OS_VIEW_ENABLE is on

/*********************************************************************
*
*       UART settings for OSView
*       If you do not want (or can not due to hardware limitations)
*       to dedicate a UART to OSView, please define it to be -1
*       Currently UART10 is supported, which is selected by definition
*       of OS_UART=1.
*       Select the UART to be used for embOSView by project settings,
*       preprocessor defines.
*/
#ifndef   OS_UART
  #define OS_UART (1)
#endif

#ifndef   OS_BAUDRATE
  #define OS_BAUDRATE (38400)
#endif

#ifndef   OS_PCLK_UART
  #define OS_PCLK_UART (OS_FSYS / 8uL) /* We select PLL/8 as clock source */
#endif

/****** End of configurable options *********************************/

#define OS_UART_USED  ((OS_VIEW_ENABLE && (OS_VIEW_USE_UART != 0)) && ((OS_UART == 1) || (OS_UART == 3)))

#if OS_UART_USED
  #define OS_COM_INIT() OS_COM_Init()
#else
  #define OS_COM_INIT()
#endif

/*********************************************************************
*
*       SFRs, types, or constants used here
*
**********************************************************************
*/

#define CLK_SEL_MAIN_OSC	 0x000Cu  /* Clock selector Main Oscillator 1 */

#define CLK_SEL_PLL0_1 		 0x0014u  /* Clock selector PLL clock / 1 */
#define CLK_SEL_PLL0_2 		 0x0015u  /* Clock selector PLL clock / 2 */
#define CLK_SEL_PLL0_4 		 0x0017u  /* Clock selector PLL clock / 4 */
#define CLK_SEL_PLL0_8  	 0x001Au  /* Clock selector PLL clock / 8 */

#define OS_TIMER_INTERRUPT_PENDING() (ICOSTM0 & (1uL << 12))

#define SFR_U32 unsigned long volatile __no_bit_access

/*********************************************************************
*
*       Local variables
*
**********************************************************************
*/

/*********************************************************************
*
*       Local functions, declarations
*
**********************************************************************
*/

static void _SetCKS_ISO0(SFR_U32* Domain, unsigned short Id, unsigned char Stop );

#if OS_UART_USED
static void _ISR_RxHandler(void);
#endif

/*********************************************************************
*
*       Local functions, implementation
*
**********************************************************************
*/

/*********************************************************************
*
*       _SetCKS_ISO0
*/
static void _SetCKS_ISO0(SFR_U32* Domain, unsigned short Id, unsigned char Stop ) {

  SFR_U32* StatusReg;

  // Calculate addr of sfr
  StatusReg = Domain + 1;

  //
  // Select clock source
  //
  PROTCMD0 = 0xa5u;         // Unlock access to sfr
  *Domain  =   (Id << 1) + Stop;
  *Domain  = ~((Id << 1) + Stop);
  *Domain  =   (Id << 1) + Stop;
  //
  // Wait until clock selection has been changed
  //
  while (*StatusReg != ((Id << 1) + 1)) {
  }
}

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_InitHW()
*
*  Initialize the hardware required for the OS to run.
*  We initialize the OS timer 0 and optionally the UART
*  for communication to embOS_View
*/
#define OS_TIMER_RELOAD (OS_PCLK_TIMER/1000)

void OS_InitHW(void) {
  //
  // Initially disable interrupts before initializing the timer
  //
  OS_IncDI();
  ICOSTM0 |= (1uL << 7);        /* Disable the OS timer interrupt     */
  //
  // Initialize OS timer 0 for embOS
  //
  OSTM0TT   = (1uL << 0);       /* Stop the timer                     */

  //
  // Setup clock source for timer from PLL (peripheral clock is limited to 48MHz)
  //
  _SetCKS_ISO0(&CKSC_032, CLK_SEL_PLL0_8, 0);
  //
  // Now configure the timer
  //
  OSTM0CTL  = (0uL << 0)        /* Disable interrupt on counter start */
            | (0uL << 1)        /* Set to interval timer mode         */
      	    ;
  OSTM0CMP  = OS_TIMER_RELOAD;  /* Set timer reload value             */
  OSTM0TS  |= (1uL << 0);       /* Start counter                      */
  //
  // Enable OS timer interrupt, lowest priority
  //
  ICOSTM0   = 15;               /* Enable interrupts with lowest priority (15) */
  //
  // Initialize the UART for embOSView
  //
  OS_COM_INIT();
  //
  // Finally restore the interrupt state
  //
  OS_DecRI();
}

/*********************************************************************
*
*       OS_Idle()
*
*  Please note:
*  This is basically the "core" of the idle loop.
*  This core loop can be changed, but:
*  The idle loop does not have a stack of its own, therefore no
*  functionality should be implemented that relies on the stack
*  to be preserved. However, a simple program loop can be programmed
*  (like toggeling an output or incrementing a counter)
*/
void OS_Idle(void) {     /* Idle loop: No task is ready to exec     */
  while (1) {
  };
}

/*********************************************************************
*
*       OS_GetTime_Cycles()
*
*  This routine is required for task-info display via embOSView or
*  high resolution time measurement functions.
*  It returns the system time in clock cycles of hardware timer.
*/
OS_U32 OS_GetTime_Cycles(void) {
  unsigned int t_cnt;
  OS_U32 time;

  t_cnt = OSTM0CNT;
  time  = OS_GetTime32();

  if (OS_TIMER_INTERRUPT_PENDING()) {
    t_cnt = OSTM0CNT;
    time++;
  }
  return (OS_PCLK_TIMER/1000) * time + OS_TIMER_RELOAD - t_cnt;
}

/*********************************************************************
*
*       OS_ConvertCycles2us()
*
*  Purpose:
*     Convert timer cycles into micro seconds.
*     Used for Profiling or high resolution time measurement.
*     If you use a clock frequency which is not a multiple of 1 MHz,
*     you may have to modify this routine in order to get proper
*     diagonstics.
*  Please note:
*     This routine is required for profiling and high resolution time
*     measurement.
*     It does not affect operation of the OS.
*/
OS_U32 OS_ConvertCycles2us(OS_U32 Cycles) {
  return Cycles/(OS_PCLK_TIMER/1000000);
}

/*********************************************************************
*
*       OS_ISR_Tick()
*       The embOS system timer interrupt handler
*/
#ifdef __ICCV850__
  #pragma vector=INTOSTM0_vector
#else
  #define OS_ISR_Tick INTOSTM0      /* Assign OS_ISR_Tick() to _INTOSTM0 from df3512_startup.850 */
#endif
__interrupt void OS_ISR_Tick(void); /* Avoid no prototype error */
__interrupt void OS_ISR_Tick(void) {
  OS_EnterNestableInterrupt();
  OS_EnterIntStack();
  OS_TICK_Handle();
  OS_LeaveIntStack();
  OS_LeaveNestableInterrupt();
}

#if (OS_UART_USED)

/*********************************************************************
*
*       Communication for embOSView V850E2S FG4
*
**********************************************************************
*/

/*  Baud rate selection = OS_PCLK_UART / 2 / baudrate */
#if (OS_BAUDRATE >= 9600u) && (OS_BAUDRATE <= 115200u)
  #define OS_UART_DIVIDER (OS_PCLK_UART / 2u / OS_BAUDRATE)
#else
  #error "Baudrate settings not supported, please modifiy OS_COM_Init()"
#endif

/*********************************************************************
*
*	Mapping of sfrs and ISR handler according to selected UART
*/
#if (OS_UART == 1)

  /******	Map ISR handler *******************************************/

  #define OS_ISR_rx  INTLMA10IR
  #define OS_ISR_tx  INTLMA10IT
  #define OS_ISR_err INTLMA10IS

  #ifdef __ICCV850__
    #define OS_ISR_ERR_VECTOR INTLMA10IS_vector
    #define OS_ISR_RX_VECTOR  INTLMA10IR_vector
    #define OS_ISR_TX_VECTOR  INTLMA10IT_vector
  #endif // __ICCV850__

#endif // (OS_UART == 1)

/*********************************************************************
*
*       OS_COM_Init()
*
*       Initialize UART for communication with embOSView
*/
void OS_COM_Init(void) {
  volatile unsigned char Dummy;

  //
  //  Initially disable interrupts
  //
  OS_IncDI();
  //
  // Set UART10 PCLK to MainOsc (16 MHz)
  //
  _SetCKS_ISO0(&CKSC_011, CLK_SEL_PLL0_8, 0);
  //
  // Initialize the UART
  //
  URTE10CTL0 = 0x00u;    // Disable UART

  URTE10CTL2 = OS_UART_DIVIDER & 0xFFFu;  // Set up baud rate generator, nor prescaler

  URTE10CTL1 = (1u << 0)    // URTEnSLIT:     1 => Tx-Interrupt on Tx complete; 0 => Tx interrupt on Tx register empty
             | (1u << 1)    // URTEnSLD:      1 => LSB first; 0 => MSB first
             | (0u << 2)    // URTEnSLG:      0 => 1 Stop bit; 1 => 2 Stop bits
             | (0u << 3)    // Reserved, always 0
             | (0u << 4)    // URTEnRDL:      0 => No inverted data on Rx
             | (0u << 5)    // URTEnTDL:      0 => No inverted data on TX
             | (0u << 6)    // URTEnSLP[1:0]: 0 => No parity
             | (1u << 8)    // URTEnCLG     : 1 => 8 data bits on Rx
             | (0u << 9)    // Reserved, b11..9 always 0
             | (0u << 12)   // URTEnBLG[2:0]: 0 => don't care when URTEnSLBM is 0
             | (0u << 15)   // URTEnSLBM:     0 => BF reception disabled
             ;

  URTE10CTL0 = (0u << 0)    // URTEnSLDC:     0 => disable data consistency check
             | (0u << 1)    // Reserved, b4..b1 always 0
             | (1u << 5)    // URTEnRXE:      1 => Enbale reception
             | (1u << 6)    // URTEnTXE:      1 => Enable transmission
             | (0u << 7)    // URTEnPW:       0 => Keep URTE stopped
             ;

  //
  // Enable UART operation
  //
  URTE10CTL0 |= (1u << 7);  // URTEnPW:       1 => Start URTE now
  //
  // Disable UART Interrupts
  //
  MKLMA10IR = 1;
  MKLMA10IT = 1;
  MKLMA10IS = 1;
  //
  // Configure Ports for UART
  //
  PMCSR0 = 0x01000100u;
  PFCE0 &= ~(1u << 8);
  PFC0  &= ~(1u << 8);
  PMSR0  = 0x01000000u;
  //
  // Configure URTE10RX on PIN  9 of Port 0
  //
  PMCSR0 = 0x02000200u;
  PFCE0 &= ~(1u << 9);
  PFC0  &= ~(1u << 9);
  PMSR0 &= 0x02000200;
  //
  // Set URTE10 RX filter to bypass mode
  //
  FCLA7CTL0=0x80u;
  //
  // Enable UART Rx interrupt
  //
  ICLMA10IRH = 0u;     // Clear interrupt
  ICLMA10IRL = 15u;    // Enable lowest priority
  //
  // Enable UART error (status) interrupt
  //
  ICLMA10ISH = 0u;     // Clear interrupt
  ICLMA10ISL = 15u;    // Enable lowest priority
  //
  // Enable UART Tx interrupt
  //
  ICLMA10ITH = 0u;     // Clear interrupt
  ICLMA10ITL = 15u;    // Enable lowest priority

  OS_DecRI();
}

/*********************************************************************
*
*       OS_COM_Send1()
*
*  Function description:
*    Transmitter Routine used by embOS for communication with
*    embOSView.
*    This routine depends on the serial interface used for communication
*    with embOSView. Therefore it is public here, so it can be modified
*    if you have to use an other UART for embOSView.
*
*  Caution:
*    NEVER call this routine from your application.
*/
void OS_COM_Send1(unsigned char c) {
  URTE10TX = c;
}

/*********************************************************************
*
*       OS_ISR_tx() transmit interrupt handler
*/
#ifdef __ICCV850__
  #pragma vector=OS_ISR_TX_VECTOR
#endif
__interrupt void OS_ISR_tx(void);
__interrupt void OS_ISR_tx(void) {
  OS_EnterInterrupt();
  OS_EnterIntStack();
  OS_OnTx();
  OS_LeaveIntStack();
  OS_LeaveInterrupt();
}

/*********************************************************************
*
*       _ISR_RxHandler() receive interrupt handler for embOSView
*/
static void _ISR_RxHandler(void) {
  OS_OnRx(URTE10RX);
}

#ifdef __ICCV850__
  #pragma vector=OS_ISR_RX_VECTOR
#endif
__interrupt void OS_ISR_rx(void);
__interrupt void OS_ISR_rx(void) {
  OS_EnterNestableInterrupt();  /* We will enable interrupts        */
  OS_EnterIntStack();           /* We will use interrupt stack      */
  _ISR_RxHandler();             /* Call to handler is required !    */
  OS_LeaveIntStack();           /* Interrupt stack switching does   */
  OS_LeaveNestableInterrupt();  /* not allow local variables in ISR */
}

/*********************************************************************
*
*       _ISR_ErrHandler() receive error interrupt handler for embOSView
*/
static void _ISR_ErrHandler(void) {
  OS_OnRx(URTE10RX);
}

#ifdef __ICCV850__
  #pragma vector=OS_ISR_ERR_VECTOR
#endif
__interrupt void OS_ISR_err(void);
__interrupt void OS_ISR_err(void) {
  OS_EnterNestableInterrupt();  /* We will enable interrupts        */
  OS_EnterIntStack();           /* We will use interrupt stack      */
  _ISR_ErrHandler();            /* Call to handler is required !    */
  OS_LeaveIntStack();           /* Interrupt stack switching does   */
  OS_LeaveNestableInterrupt();  /* not allow local variables in ISR */
}

#else  /* No supported UART selected; use dummy routines            */

/*********************************************************************
*
*       Dummy communication routines
*
**********************************************************************
*/

void OS_COM_Send1(OS_U8 c) {
  OS_USEPARA(c);           /* Avoid compiler warning */
  OS_COM_ClearTxActive();  /* Release transmitter    */
}
#endif

/********************************************************************/

/*********************************************************************
*
*       __low_level_init()
*
*  Function description:
*    This function is called from the startup code directly after
*    the system start, before any segments are initialized.
*    Therfore, the function should not rely on any variables or data.
*
*    The function may initialize a high speed CPU clock to speed up
*    booting the system.
*    Any othe initialization which might be required directly after
*    reset may be added here.
*
*  Returns:
*    1 to force segment initialization in the startup code.
*
**********************************************************************
*/

unsigned char __low_level_init(void) {
  unsigned int TempReg;
  //
  // Start main oscillator when it is not running
  //
  if(MOSCS == 0uL) {
    //
    // Set MainOsc stabilization time register
    //
    MOSCST   = 0x00000000FuL;        // Max value (16.38ms)
    TempReg  = MOSCE | (1uL << 0);
    PROTCMD2 = 0xa5u;                // Unlock access to sfr
    MOSCE    =  TempReg;
    MOSCE    = ~TempReg;
    MOSCE    =  TempReg;
  }
  //
  // Wait until MOSC is stable
  //
  while (MOSCS != 0x00000007uL) {
  }
#if OS_INIT_PLL // May be disabled by project settings to allow usage of simulator
  //
  // Initialize PLL to speed up CPU clock
  //
  if (PLLS0 != 0x0000000uL) {       // PLL already running?
    TempReg  = PLLE0 | (1uL << 1);  // Stop Pll
    PROTCMD2 = 0xa5u;               // Unlock access to sfr
    PLLE0    =  TempReg;
    PLLE0    = ~TempReg;
    PLLE0    =  TempReg;
    //
    // Wait until Pll is stopped
    //
    while (PLLS0 != 0x0u) {
    }
  }
  //
  // Setup Pll to generate 64 MHz CPU clock, assuming 16 MHz external crystal
  //
  PLLC0 = 0x000207;   // Pr = 2, Nr = 8(-1) => (16 Mhz * 8) / 2 = 64 MHz
  //
  // Start Pll
  //
  TempReg  = PLLE0 | (1uL << 0);
  PROTCMD2 = 0xa5u;                 // Unlock access to sfr
  PLLE0    =  TempReg;
  PLLE0    = ~TempReg;
  PLLE0    =  TempReg;
  //
  // Wait until Pll is stable
  //
  while (PLLS0 != 0x7u) {
  }
  //
  // Set clock for CPU clock domain to PLL
  //
  _SetCKS_ISO0(&CKSC_000, CLK_SEL_PLL0_1, 0);
#endif
  //
  // Add more initialization here if required
  //

  return 1;  // Force initialization of data segments
}

/****** End Of File *************************************************/
