/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2012  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 3.86k                                            *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File    : RTOSInit_V850E2_MN4.c
          for RENESAS V850E2_MN4 CPUs

Purpose : Initializes and handles the hardware for the OS as far
          as required by the OS.
          Feel free to modify this file acc. to your
          target system.

-------- END-OF-HEADER -----------------------------------------------
*/

#include "RTOS.h"
#ifndef __ICCV850__
#include "df3512_0.h"
#else
#include "io70f3512.h"
#endif
/*********************************************************************
*
*       Configuration
*
**********************************************************************

  Define clock frequency for CPU: used to initialize embOS timer
  and baudrate for optional UART for embOSView

  Set UART and Baudrate for embOSView
  If you do not want (or can not due to hardware limitations) to
  dedicate a UART to embOSView, please define OS_UART to -1

*/
#ifndef   OS_FSYS
  #define OS_FSYS 200000000uL  /* default is 200 MHz, depends on external crystal */
#endif

#ifndef   OS_PCLK_TIMER
  #define OS_PCLK_TIMER  (OS_FSYS / 3uL) /* Fixed prescaler for PCLK */
#endif

/*********************************************************************
*
*       Configuration of communication to OSView
*/
#ifndef   OS_VIEW_ENABLE            // Global enable of communication
  #define OS_VIEW_ENABLE    (1)     // Default: on
#endif

#ifndef   OS_VIEW_USE_UART          // If set, UART will be used
  #define OS_VIEW_USE_UART  (1)     // Default: 1 => Uart is used
#endif                              // if OS_VIEW_ENABLE is on

/*********************************************************************
*
*       UART settings for OSView
*       If you do not want (or can not due to hardware limitations)
*       to dedicate a UART to OSView, please define it to be -1
*       Currently UARTF1 and UARTF3 are supported.
*       Select the UART to be used for embOSView by project settings, preprocessor defines
*       Either define OS_UART=1 or OS_UART=3 to use one of the supported UARTS.
*/
#ifndef   OS_UART
  #define OS_UART (3)
#endif

#ifndef   OS_BAUDRATE
  #define OS_BAUDRATE (38400)
#endif

#ifndef   OS_PCLK_UART
  #define OS_PCLK_UART  (OS_FSYS / 3uL) /* Fixed prescaler for PCLK */
#endif

/****** End of configurable options *********************************/

#define OS_UART_USED  ((OS_VIEW_ENABLE && (OS_VIEW_USE_UART != 0)) && ((OS_UART == 1) || (OS_UART == 3)))

#if OS_UART_USED
  #define OS_COM_INIT() OS_COM_Init()
#else
  #define OS_COM_INIT()
#endif

/*********************************************************************
*
*       Locals
*
**********************************************************************
*/

/*********************************************************************
*
*       Globals
*
**********************************************************************
*/

/*********************************************************************
*
*       Local functions
*
**********************************************************************
*/

#define OS_TIMER_INTERRUPT_PENDING() (ICOSTM0 & (1uL << 12))

/*********************************************************************
*
*       OS_InitHW()
*
*  Initialize the hardware required for the OS to run.
*  We initialize the OS timer 0 and optionally the UART
*  for communication to embOS_View
*/
#define OS_TIMER_RELOAD (OS_PCLK_TIMER/1000)

void OS_InitHW(void) {
//  WDTM2    = 0x0f;              /* Stop watchdog        */
  //
  // Initially disable interrupts before initializing the timer
  //
  OS_IncDI();
  ICOSTM0 |= (1uL << 7);        /* Disable the OS timer interrupt     */
  //
  // Initialize OS timer 0 for embOS
  //
  OSTM0TT   = (1uL << 0);       /* Stop the timer                     */
  OSTM0CTL  = (0uL << 0)        /* Disable interrupt on counter start */
            | (0uL << 1)        /* Set to interval timer mode         */
	    ;
  OSTM0TOE  = 0;                /* Disable timer output               */
  OSTM0CMP  = OS_TIMER_RELOAD;  /* Set timer reload value             */
  OSTM0TS  |= (1uL << 0);       /* Start counter                      */
  //
  // Enable OS timer interrupt, lowest priority
  //
  ICOSTM0   = 15;               /* Enable interrupts with lowest priority (15) */
  //
  // Initialize the UART for embOSView
  //
  OS_COM_INIT();
  //
  // Finally restore the interrupt state
  //
  OS_DecRI();
}

/*********************************************************************
*
*       OS_Idle()
*
*  Please note:
*  This is basically the "core" of the idle loop.
*  This core loop can be changed, but:
*  The idle loop does not have a stack of its own, therefore no
*  functionality should be implemented that relies on the stack
*  to be preserved. However, a simple program loop can be programmed
*  (like toggeling an output or incrementing a counter)
*/

void OS_Idle(void) {     /* Idle loop: No task is ready to exec     */
  while (1) {
  };
}

/*********************************************************************
*
*       OS_GetTime_Cycles()
*
*  This routine is required for task-info display via embOSView or
*  high resolution time measurement functions.
*  It returns the system time in clock cycles of hardware timer.
*/
OS_U32 OS_GetTime_Cycles(void) {
  unsigned int t_cnt;
  OS_U32 time;

  t_cnt = OSTM0CNT;
  time  = OS_GetTime32();

  if (OS_TIMER_INTERRUPT_PENDING()) {
    t_cnt = OSTM0CNT;
    time++;
  }
  return (OS_PCLK_TIMER/1000) * time + OS_TIMER_RELOAD - t_cnt;
}

/*********************************************************************
*
*       OS_ConvertCycles2us()
*
*  Purpose:
*     Convert timer cycles into micro seconds.
*     Used for Profiling or high resolution time measurement.
*     If you use a clock frequency which is not a multiple of 1 MHz,
*     you may have to modify this routine in order to get proper
*     diagonstics.
*  Please note:
*     This routine is required for profiling and high resolution time
*     measurement.
*     It does not affect operation of the OS.
*/
OS_U32 OS_ConvertCycles2us(OS_U32 Cycles) {
  return Cycles/(OS_PCLK_TIMER/1000000);
}

/*********************************************************************
*
*       OS_ISR_Tick()
*       The embOS system timer interrupt handler
*/
#ifdef __ICCV850__
  #pragma vector=INTOSTM0_vector
#else
  #define OS_ISR_Tick INTOSTM0      /* Assign OS_ISR_Tick() to _INTOSTM0 from df3512_startup.850 */
#endif
__interrupt void OS_ISR_Tick(void); /* Avoid no prototype error */
__interrupt void OS_ISR_Tick(void) {
  OS_EnterNestableInterrupt();
  OS_EnterIntStack();
  OS_TICK_Handle();
  OS_LeaveIntStack();
  OS_LeaveNestableInterrupt();
}

#if (OS_UART_USED)

/*********************************************************************
*
*       Communication for embOSView V850E2 MN4
*
**********************************************************************
*/

/*  Baud rate select = OS_FSYS/2/baudrate */
#if (OS_BAUDRATE >= 9600) && (OS_BAUDRATE <= 115200)
  #define OS_UART_DIVIDER (OS_PCLK_UART / 2 / OS_BAUDRATE)
#else
  #error "Baudrate settings not supported, please modifiy OS_COM_Init()"
#endif

/*********************************************************************
*
*	Mapping of sfrs and ISR handler according to selected UART
*/
#if (OS_UART == 1)

/******	Map sfrs for UART1F to symbolic names ***********************/
#define URTJxCTL0  URTJ1CTL0
#define URTJxCTL1  URTJ1CTL1
#define URTJxCTL2  URTJ1CTL2
#define URTJxTRG   URTJ1TRG
#define URTJxSTR0  URTJ1STR0
#define URTJxSTR1  URTJ1STR1
#define URTJxSTC   URTJ1STC
#define URTJxFCTL0 URTJ1FCTL0
#define URTJxFCTL1 URTJ1FCTL1
#define URTJxFSTR0 URTJ1FSTR0
#define URTJxFSTR1 URTJ1FSTR1
#define URTJxFSTC  URTJ1FSTC
#define URTJxFTX   URTJ1FTX
#define URTJxFRX   URTJ1FRX

#define ICCSIHxIR  ICCSIH1IR
#define ICCSIHxIC  ICCSIH1IC

/******	Assign Port to UART1F ***************************************/

#define PMC_UART_RX      PMC13
#define PFC_UART_RX      PFC13
#define PFCE_UART_RX     PFCE13
#define PM_UART_RX       PM13
#define PIS_UART_RX      PIS13
#define PISE_UART_RX     PISE13
#define PMC_UART_RX_BIT  5

#define PMC_UART_TX      PMC13
#define PFC_UART_TX      PFC13
#define PFCE_UART_TX     PFCE13
#define PM_UART_TX       PM13
#define PODC_UART_TX     PODC13
#define PMC_UART_TX_BIT  4

/******	Map ISR handler *********************************************/
#define OS_ISR_rx  INTCSIH1IR
#define OS_ISR_tx  INTCSIH1IC

#ifdef __ICCV850__
#define OS_ISR_RX_VECTOR INTCSIH1IR_vector
#define OS_ISR_TX_VECTOR INTCSIH1IC_vector
#endif // __ICCV850__

#elif (OS_UART == 3)
/******	Map sfrs for UART3F to symbolic names ***********************/
#define URTJxCTL0  URTJ3CTL0
#define URTJxCTL1  URTJ3CTL1
#define URTJxCTL2  URTJ3CTL2
#define URTJxTRG   URTJ3TRG
#define URTJxSTR0  URTJ3STR0
#define URTJxSTR1  URTJSTR1
#define URTJxSTC   URTJ3STC
#define URTJxFCTL0 URTJ3FCTL0
#define URTJxFCTL1 URTJ3FCTL1
#define URTJxFSTR0 URTJ3FSTR0
#define URTJxFSTR1 URTJ3FSTR1
#define URTJxFSTC  URTJ3FSTC
#define URTJxFTX   URTJ3FTX
#define URTJxFRX   URTJ3FRX

/******	Assign Port to UART1F ***************************************/

#define PMC_UART_RX      PMC4
#define PFC_UART_RX      PFC4
#define PFCE_UART_RX     PFCE4
#define PM_UART_RX       PM4
#define PIS_UART_RX      PIS4
#define PISE_UART_RX     PISE4
#define PISA_UART_RX     PISE4
#define PMC_UART_RX_BIT  6

#define PMC_UART_TX      PMC4
#define PFC_UART_TX      PFC4
#define PFCE_UART_TX     PFCE4
#define PM_UART_TX       PM4
#define PODC_UART_TX     PODC4
#define PDSC_UART_TX     PDSC4

#define PMC_UART_TX_BIT  7

/******	Map ISR handler *********************************************/
#define OS_ISR_rx  INTCSIH3IR
#define OS_ISR_tx  INTCSIH3IC
#define ICCSIHxIR  ICCSIH3IR
#define ICCSIHxIC  ICCSIH3IC

#ifdef __ICCV850__
#define OS_ISR_RX_VECTOR INTCSIH3IR_vector
#define OS_ISR_TX_VECTOR INTCSIH3IC_vector
#endif // __ICCV850__

#endif // (OS_UART==3)

/*********************************************************************
*
*       OS_COM_Init()
*
*       Initialize UART for communication with embOSView
*/
void OS_COM_Init(void) {
  volatile unsigned char Dummy;

  /*  Initially disable interrupts */
  OS_DI();
  //
  // Disable UART
  //
  ICCSIHxIR |= (1uL << 7);  // Disable Rx interrupt
  ICCSIHxIC |= (1uL << 7);  // Disable Tx interrupt
  URTJxCTL0 = 0;
  //
  // Setup mode 8N1
  //
  URTJxCTL1 = (0uL << 0)    // Tx interrupt on TxEmpty
            | (1uL << 1)    // LSB first
            | (0uL << 2)    // 1 Stop bit
            | (0uL << 3)    // Reserved, write 0
            | (0uL << 4)    // Non inverted receive data level
            | (0uL << 5)    // Non inverted transmit data level
            | (0uL << 6)    // b7..b6:  Parity select. 0 => NO parity
            | (1uL << 8)    // 8 bit data length
            | (0uL << 9)    // b11..b9:  Reserved, write 0
            | (0uL << 12)   // b14..b12: BF length, Not used, don't care
            | (0uL << 15)   // BF reception disabled
            ;
  //
  // Setup baudrate
  //
  URTJxCTL2 = (OS_UART_DIVIDER & 0xFFF)
            | (0uL << 12)   // Reserved, write 0
            | (0uL << 13)   // b15..b13: Prescaler, set to 0 for most baudrates
            ;
  URTJxSTC  = (1uL << 1)    // Clear Framing error
            | (1uL << 2)    // Clear Parity error
            | (1uL << 3)    // Clear data consistency error
            | (1uL << 4)    // Clear BF reception successfull flag
            ;
  //
  // Setup FIFO (not used)
  //
  URTJxFCTL0 = (0uL   << 0) // Tx FIFO interrupt level
             | (0xFuL << 8) // Rx FIFO interrupt level
             ;
  URTJxFCTL1 = 0;           // Disable Rx timeout
  URTJxFSTC  = (1uL << 0)   // Clear Rx FIFO counter
             | (1uL << 1)   // Clear Tx FIFO counter
             | (0uL << 2)   // b4..b2: Reserved, write 0
             | (1uL << 5)   // Clear Overrun error
             | (1uL << 6)   // Clear Overflow error
             | (1uL << 7)   // Clear timeout error
             ;
  //
  // Enable UART, enable Rx, Tx
  //
  URTJxCTL0 = (1uL << 7) // Enable UART
            | (1uL << 6) // Enable Tx
            | (1uL << 5) // Enable Rx
            ;
  //
  // Setup port for UART, Rx input, Tx output
  //
  PMC_UART_RX  |=  (1uL << PMC_UART_RX_BIT);  // Set Port alternate function mode for Rx
  PFC_UART_RX  |=  (1uL << PMC_UART_RX_BIT);  // Set Port alternate function mode for Rx
  PFCE_UART_RX &= ~(1uL << PMC_UART_RX_BIT);  // Set Port alternate function mode for Rx
  PM_UART_RX   |=  (1uL << PMC_UART_RX_BIT);  // Set Port to input for Rx
  PIS_UART_RX  |=  (1uL << PMC_UART_RX_BIT);  // Activate Port input buffer
  PISE_UART_RX &= ~(1uL << PMC_UART_RX_BIT);
#ifdef PISA_UART_RX
  PISA_UART_RX &= ~(1uL << PMC_UART_RX_BIT);
#endif
#ifdef PU_UART_RX
  PISA_UART_RX &= ~(1uL << PMC_UART_RX_BIT);
#endif
  PMC_UART_TX  |=  (1uL << PMC_UART_TX_BIT);  // Set Port alternate function mode for Tx
  PFC_UART_TX  |=  (1uL << PMC_UART_TX_BIT);  // Set Port alternate function mode for Tx
  PFCE_UART_TX &= ~(1uL << PMC_UART_TX_BIT);  // Set Port alternate function mode for Tx
  PM_UART_TX   &= ~(1uL << PMC_UART_TX_BIT);  // Set Port to output for Tx
  PODC_UART_TX &= ~(1uL << PMC_UART_TX_BIT);  // Set output mode to push-pull
#ifdef PDSC_UART_TX
  PDSC_UART_TX &= ~(1uL << PMC_UART_TX_BIT);  // Set output mode to push-pull
#endif
  //
  // Enable UART Rx Tx interrupts
  //
  ICCSIHxIR = 15;  // Enable Rx interrupt, lowest priority
  ICCSIHxIC = 15;  // Enable Tx interrupt, lowest priority

  OS_RestoreI();
}

/*********************************************************************
*
*       OS_COM_Send1
*

Purpose: Transmitter Routine used by embOS for communication with
         embOSView.

Caution: NEVER call this routine from your application.

This routine depends on the serial interface used for communication
with embOSView. Therefore it is public here, so it can be modified
if you have to use an other UART for embOSView.
*/
void OS_COM_Send1(unsigned char c) {
  URTJxFTX = c;
}

/*********************************************************************
*
*       OS_ISR_tx transmit interrupt handler
*/
#ifdef __ICCV850__
  #pragma vector=OS_ISR_TX_VECTOR
#endif
__interrupt void OS_ISR_tx(void);
__interrupt void OS_ISR_tx(void) {
  OS_EnterInterrupt();
  OS_EnterIntStack();
  OS_OnTx();
  OS_LeaveIntStack();
  OS_LeaveInterrupt();
}

/*********************************************************************
*
*       OS_ISR_rx receive interrupt handler
*/
static void _ISR_RxHandler(void) {
  OS_U16 RxData;

  RxData = URTJxFRX;
  if ((RxData & (3 << 14)) == 0) { // No error, process current data
    OS_OnRx((OS_U8) RxData);
  } else {
    //
    // Reset error state in UART Status register
    //
    URTJxSTC = (1uL << 1) | (1uL << 2);  // Reset Framing error, reset parity error
  }
}


#ifdef __ICCV850__
  #pragma vector=OS_ISR_RX_VECTOR
#endif
__interrupt void OS_ISR_rx(void);
__interrupt void OS_ISR_rx(void) {
  OS_EnterNestableInterrupt();  /* We will enable interrupts        */
  OS_EnterIntStack();           /* We will use interrupt stack      */
  _ISR_RxHandler();             /* Call to handler is required !    */
  OS_LeaveIntStack();           /* Interrupt stack switching does   */
  OS_LeaveNestableInterrupt();  /* not allow local variables in ISR */
}

#else  /* No supported UART selected; use dummy routines            */

/*********************************************************************
*
*       Dummy communication routines
*
**********************************************************************
*/

void OS_COM_Send1(OS_U8 c) {
  OS_USEPARA(c);           /* Avoid compiler warning */
  OS_COM_ClearTxActive();  /* Release transmitter    */
}
#endif

/****** End Of File *************************************************/
