/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2012  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 3.86k                                            *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File   : RTOSInit_V850ES_SG2.c
Purpose: Initializes and handles the hardware for embOS as far
         as required by embOS for V850ES SG2.
         Feel free to modify this file acc. to your
         target system.
-------- END-OF-HEADER -----------------------------------------------
*/

#include "RTOS.H"
#ifdef __ghs                    /* SFR file used with Green Hills   */
  #include <DF3281.h>
#else
  #include "io70f3281y.h"       /* SFR file delivered from IAR      */
#endif

/*********************************************************************
*
*       Configuration
*
**********************************************************************

  Define clock frequency for CPU: used to initilize embOS timer
  and baudrate for optional UART for embOSView

  Set UART and Baudrate for embOSView
  If you do not want (or can not due to hardware limitations) to
  dedicate a UART to embOSView, please define OS_UART to -1

*/

#ifndef   OS_FSYS
  #define OS_FSYS 20000000  /* default is 20 MHz, may depend on PLL */
#endif
#ifndef   OS_UART
  #define OS_UART 0
#endif
#ifndef   OS_BAUDRATE
  #define OS_BAUDRATE 38400L
#endif

#ifndef   OS_PCLK_TIMER
  #define OS_PCLK_TIMER OS_FSYS
#endif

#ifndef   OS_PCLK_UART
  #define OS_PCLK_UART OS_FSYS
#endif

#ifndef   INIT_PLL
  #define INIT_PLL 0
#endif

/*********************************************************************
*
*       Locals
*
**********************************************************************
*/

/*********************************************************************
*
*       _InitPLL
*       may be used and called from OS_InitHW() if required.
*       Please carefully examine whether this function can be used
*       with your hardware or has to be modified.
*/

#if INIT_PLL
static void _InitPLL(void) {
  PRCMD   = 0x55;                     /* unlock */
  PCC     = 0x00;                     /* Set operation clock, main clock, full speed */
  PLLCTL &= ~(1 << 1);                /* Deselect PLL  */
  PRCMD   = 0x55;                     /* unlock CKC    */
  CKC     = 0x0A;                     /* Multiply by 4 */
  while (LOCKR & (1 << 0)) {          /* Wait until PLL is stable */
  }
  PLLCTL |= (1 << 1);                 /* Select PLL */
}
#endif

/*********************************************************************
*
*       Globals
*
**********************************************************************
*/

/*********************************************************************
*
*       OS_InitHW
*
*  Initialize the hardware required for the OS to run. This will work
*  on any target hardware, but may have to be tailored a little
*  (regarding the clock frequency). Of course the same holds true
*  if for some reason you choose to use an other timer.
*/

#define TIMER_RELOAD (OS_PCLK_TIMER/1000 - 1)

void OS_InitHW(void) {
  #if INIT_PLL
    _InitPLL();
  #endif
  TP0CTL0  = 0x00;           /* stop timer TMP0 for initializatio           */
  TP0CCR0  = TIMER_RELOAD;   /* compare register 0 set to 1ms resolution    */
  TP0CTL1  = 0x00;           /* interval timer mode                         */
  TP0IOC0  = 0x00;           /* timer output (not used)                     */
  TP0IOC1  = 0x00;           /* timer capture trigger input (not used)      */
  TP0IOC2  = 0x00;           /* timer event count input (not used)          */
  TP0OPT0  = 0x00;           /* cc register selection, ovf (not used)       */
  TP0CTL0  = 0x80;           /* enable timer, prescale 1                    */
  TP0CCIC0 = 0x07;           /* enable timerp0 interrupt0, lowest priority  */
  OS_COM_Init();
}

/*********************************************************************
*
*       Idle Loop  (OS_Idle)
*
*  Please note:
*  This is basically the "core" of the idle loop.
*  This core loop can be changed, but:
*  The idle loop does not have a stack of its own, therefore no
*  functionality should be implemented that relies on the stack
*  to be preserved. However, a simple program loop can be programmed
*  (like toggeling an output or incrementing a counter)
*/

void OS_Idle(void) {     /* Idle loop: No task is ready to exec     */
  while (1);
}

/*********************************************************************
*
*       Get time [cycles]
*
*  This routine is required for task-info via embOS Viewer or
*  high resolution time measurement functions.
*  It returns the system time in clock cycles of hardware timer.
*/

#define HW_TIMER TP0CNT
#define HW_TIMER_INTERRUPT_PENDING() (TP0CCIC0 & 0x80)

OS_U32 OS_GetTime_Cycles(void) {
  unsigned int t_cnt = HW_TIMER;
  OS_U32 time        = OS_Time;
  if (HW_TIMER_INTERRUPT_PENDING()) {
    t_cnt = HW_TIMER;
    if (t_cnt == TIMER_RELOAD) {
      t_cnt = 0;
    }
    time++;
  }
  return (OS_PCLK_TIMER/1000)*time + t_cnt;
}

/*********************************************************************
*
*       OS_ConvertCycles2us
*
*  Purpose:
*     Convert timer cycles into micro seconds.
*     Used for Profiling or high resolution time measurement.
*     If you use a clock frequency which is not a multiple of 1 MHz,
*     you may have to modify this routine in order to get proper
*     diagonstics.
*  Please note:
*     This routine is required for profiling and high resolution time
*     measurement.
*     It does not affect operation of the OS.
*/

OS_U32 OS_ConvertCycles2us(OS_U32 Cycles) {
  return Cycles/(OS_PCLK_TIMER/1000000);
}

/*********************************************************************
*
*       OS_Tick interrupt
*/

#ifdef __ghs                    /* This declaration for Green Hills */
__interrupt void OS_ISR_Tick(void)
  #pragma intvect OS_ISR_Tick 0x170
#else                           /* This for IAR compiler            */
  #pragma vector=INTTP0CC0_vector
__interrupt void OS_ISR_Tick(void); /* Avoid prototype error */
__interrupt void OS_ISR_Tick(void)
#endif
{
  OS_EnterNestableInterrupt();
  OS_HandleTick();
  OS_LeaveNestableInterrupt();
}

#if (OS_UART == 0)

/*********************************************************************
*
*       Communication for embOSView V850ES/SG2 UARTA 0
*
**********************************************************************
*/

/*  baud = OS_FSYS/[2^(BAUD_M) * BAUD_K] */
#if (OS_PCLK_UART == 20000000)
  #define OS_UART_DIVIDER 0x41
  #if   (OS_BAUDRATE == 38400)
    #define OS_UART_PRESCALE 0x02
  #elif (OS_BAUDRATE == 19200)
    #define OS_UART_PRESCALE 0x03
  #elif (OS_BAUDRATE == 9600)
    #define OS_UART_PRESCALE 0x04
  #elif (OS_BAUDRATE == 4800)
    #define OS_UART_PRESCALE 0x05
  #elif (OS_BAUDRATE == 2400)
    #define OS_UART_PRESCALE 0x06
  #else
    #define OS_BAUDRATE_ERROR 1
  #endif
#elif (OS_PCLK_UART == 16000000)
  #define OS_UART_DIVIDER 0x0D
  #if   (OS_BAUDRATE == 38400)
    #define OS_UART_PRESCALE 0x04
  #elif (OS_BAUDRATE == 19200)
    #define OS_UART_PRESCALE 0x05
  #elif (OS_BAUDRATE == 9600)
    #define OS_UART_PRESCALE 0x06
  #elif (OS_BAUDRATE == 4800)
    #define OS_UART_PRESCALE 0x07
  #elif (OS_BAUDRATE == 2400)
    #define OS_UART_PRESCALE 0x08
  #else
    #define OS_BAUDRATE_ERROR 1
  #endif
#elif (OS_PCLK_UART == 10000000)
  #define OS_UART_DIVIDER 0x41
  #if   (OS_BAUDRATE == 38400)
    #define OS_UART_PRESCALE 0x01
  #elif (OS_BAUDRATE == 19200)
    #define OS_UART_PRESCALE 0x02
  #elif (OS_BAUDRATE == 9600)
    #define OS_UART_PRESCALE 0x03
  #elif (OS_BAUDRATE == 4800)
    #define OS_UART_PRESCALE 0x04
  #elif (OS_BAUDRATE == 2400)
    #define OS_UART_PRESCALE 0x05
  #else
    #define OS_BAUDRATE_ERROR 1
  #endif
#else
  #define OS_BAUDRATE_ERROR 1
#endif

#ifdef OS_BAUDRATE_ERROR
  #error "Baudrate settings not supported, please define OS_UART_PRESCALE and OS_UART_DIVIDER"
#endif

/*********************************************************************
*
*       OS_COM_Init()
*
*       Initialize UART for communication with embOSView
*/

void OS_COM_Init(void) {
  volatile unsigned char Dummy;
  /*  Initially disable interrupts */
  OS_DI();                                       /* Disable interrupts   */
  UA0RIC   = 0x47;                               /* Disable Rx interrupt, set lowest priority */
  UA0TIC   = 0x47;                               /* Disable Tx interrupt, set lowest priority */
  /*  Initially disable UART A0 */
  CB4CTL0  = 0x01;                               /* Disable CSI 4 which shares same port */
  UA0CTL0 &=  ~(1 << 6);                         /* Initially disable Tx */
  UA0CTL0 &=  ~(1 << 5);                         /* disable Rx */
  UA0CTL0 &=  ~(1 << 7);                         /* disable Uart */
  /*  Setup UART port functions for UART A0 */
  PFC3    &= ~((1 << 2) | (1 << 1) | (1 << 0));  /* Port function 3.0 + 3.1 + 3.2 set to UARTA0 */
  PMC3    &=  ~(1 << 2);                         /* Port mode 3.2 set to I/O port (sclk not used) */
  PMC3    |=  ((1 << 1) | (1 << 0));             /* Port mode 3.1 + 3.0 set to UART (Rx Tx) */
  PM3     |=   (1 << 1);                         /* Port 3.1 (Rx) set to input mode  */
  PM3     &=  ~(1 << 0);                         /* Port 3.0 (Tx) set to output mode */
  /*  Setup UART control registers */
  UA0CTL0 =  (1 << 4)                            /* LSB first */
            |(0 << 3)                            /* No parity during Tx */
            |(0 << 2)                            /* No parity during Rx */
            |(1 << 1)                            /* 8 bits per character */
            |(0 << 0);                           /* 1 Stop bit */
  UA0OPT0  = 0x14;                               /* default value */
  UA0STR   = 0x00;                               /* Reset Transmit Flag and all errors */
  Dummy    = UA0RX;                              /* Dummy read to clear Rx buffer */
  /*  Initialize Baudrate generator */
  UA0CTL1  = OS_UART_PRESCALE;
  UA0CTL2  = OS_UART_DIVIDER;
  /*  Enable UART, Rx, Tx */
  UA0CTL0 |=  (1 << 7);                          /* enable Uart     */
  /*  Wait two base clock cycles                                    */
  UA0CTL0 |=  (1 << 6);                          /* enable Tx       */
  UA0CTL0 |=  (1 << 5);                          /* enable Rx       */
  UA0RIC  &= ~0x40;                              /* enable RX ISR   */
  UA0TIC  &= ~0x40;                              /* enable TX ISR   */
  OS_RestoreI();
}

/*********************************************************************
*
*       OS_COM_Send1
*

Purpose: Transmitter Routine used by embOS for communication with
         embOSView.

Caution: NEVER call this routine from your application.

This routine depends on the serial interface used for communication
with embOSView. Therefore it is public here, so it can be modified
if you have to use an other UART for embOSView.
*/

void OS_COM_Send1(unsigned char c) {
  UA0TX = c;
}

/*********************************************************************
*
*       OS_ISR_tx transmit interrupt handler
*/

#ifdef __ghs                    /* This declaration for Green Hills */
__interrupt void OS_ISR_tx(void)
  #pragma intvect OS_ISR_tx 0x320
#else                           /* This for IAR compiler            */
  #pragma vector = INTUA0T_vector
__interrupt void OS_ISR_tx(void);   /* Avoid prototype error */
__interrupt void OS_ISR_tx(void)
#endif
{
  OS_EnterInterrupt();
  OS_EnterIntStack();
  OS_OnTx();
  OS_LeaveIntStack();
  OS_LeaveInterrupt();
}

/*********************************************************************
*
*       OS_ISR_rx receive interrupt handler
*/

static void OS_ISR_Rx_Handler(void) {
  volatile unsigned char Dummy;
  if ((UA0STR & 0x07) == 0) {   /* No error                         */
    OS_OnRx(UA0RX);             /* Process data byte                */
  } else {
    UA0STR &= ~0x07;            /* Reset error                      */
    Dummy   = UA0RX;            /* discard Byte                     */
  }
}

#ifdef __ghs                    /* This declaration for Green Hills */
__interrupt void OS_ISR_rx(void)
  #pragma intvect OS_ISR_rx 0x310
#else                           /* This for IAR compiler            */
  #pragma vector = INTUA0R_vector
__interrupt void OS_ISR_rx(void);   /* Avoid prototype error */
__interrupt void OS_ISR_rx(void)
#endif
{
  OS_EnterNestableInterrupt();  /* We will enable interrupts        */
  OS_EnterIntStack();           /* We will use interrupt stack      */
  OS_ISR_Rx_Handler();          /* Call to handler is required !    */
  OS_LeaveIntStack();           /* Interrupt stack switching does   */
  OS_LeaveNestableInterrupt();  /* not allow local variables in ISR */
}

#else  /* No supported UART selected; use dummy routines            */

/*********************************************************************
*
*       Dummy communication routines
*
**********************************************************************
*/

void OS_COM_Init(void) {}
void OS_COM_Send1(OS_U8 c) {
  OS_USEPARA(c);           /* Avoid warning       */
  OS_COM_ClearTxActive();  /* Release transmitter */
}
#endif

/****** End of file **************************************************/
