/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2012  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 3.86k                                            *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File        : RTOSInit.c
Purpose     : Initializes and handles the hardware for embOS as far
              as required by embOS for V850 IA1.
              Feel free to modify this file acc. to your
              target system.
------------- END-OF-HEADER ------------------------------------------
*/

#include "RTOS.H"
#include <io70f3116.h>    /* SFR file delivered from IAR      */

/*********************************************************************
*
*       Configuration
*
**********************************************************************

  Define clock frequency for CPU: used to initilize embOS timer
  and baudrate for optional UART for embOSView

  Set UART and Baudrate for embOSView
  If you do not want (or can not due to hardware limitations) to
  dedicate a UART to embOSView, please define OS_UART to -1

*/

#ifndef   OS_FSYS
  #define OS_FSYS 40000000     /* default is 40 MHz                 */
#endif
#ifndef   OS_UART
  #define OS_UART 1
#endif
#ifndef   OS_BAUDRATE
  #define OS_BAUDRATE 38400L
#endif
#define TIMER_PRESCALE  4      /* prescaler for timer frequency     */
#define SYS_CLK 1000           /* 1/SYS_CLK sec. is OS Timer        */

/*********************************************************************
*
*       OS_InitHW
*
**********************************************************************

Initialize the hardware required for embOS to run. This will work
on any target hardware, but may have to be tailored a little
(regarding the clock frequency). Of course the same holds true
if for some reason you choose to use an other timer.
*/

void OS_InitHW(void) {
  #define TIMER_RELOAD OS_FSYS/TIMER_PRESCALE/SYS_CLK
  OS_DI();
  PHCMD = 0x55;
  CKC   = 0x07;
  __asm("nop");
  __asm("nop");
  __asm("nop");
  __asm("nop");
  OS_RestoreI();
  TMC30   = 0x01;          /* initialize / stop timer 3             */
  TMC31   = 0x29;          /* use CC30 as compare and clear TM3 after match */
  CC30    = TIMER_RELOAD;
  TMC30   = 0x03;          /* start timer with f_CLK/2 (OS_FSYS/4)  */
  CC3IC0  = 0x07;          /* enable INTCC30 (vector 0x260)         */
  OS_COM_Init();
}

/*********************************************************************
*
*       Idle Loop  (OS_Idle)
*
**********************************************************************

 Please note:
 This is basically the "core" of the idle loop.
 This core loop can be changed, but:
 The idle loop does not have a stack of its own, therefore no
 functionality should be implemented that relies on the stack
 to be preserved. However, a simple program loop can be programmed
 (like toggeling an output or incrementing a counter)
*/
void OS_Idle(void) {     /* Idle loop: No task is ready to exec     */
  while (1) {
  }
}

/*********************************************************************
*
*       Get time [cycles]
*
**********************************************************************

    This routine is required for task-info via embOS Viewer or
    high resolution time measurement functions.
    It returns the system time in clock cycles of hardware timer.
*/

#define HW_TIMER TM3
#define HW_TIMER_INTERRUPT_PENDING() (CC3IC0 & 0x80)

OS_U32 OS_GetTime_Cycles(void) {
  unsigned int t_cnt = HW_TIMER;
  OS_U32 time        = OS_Time;
  if (HW_TIMER_INTERRUPT_PENDING()) {
    t_cnt = HW_TIMER;
    if (t_cnt == TIMER_RELOAD) {
      t_cnt =0;
    }
    time++;
  }
  return (OS_FSYS/TIMER_PRESCALE/SYS_CLK)*time  + t_cnt;
}

/*********************************************************************
*
*       OS_ConvertCycles2us
*
**********************************************************************

Purpose:
   Convert timer cycles into micro seconds.
   Used for Profiling or high resolution time measurement.
   If you use a clock frequency which is not a multiple of 1 MHz,
   you may have to modify this routine in order to get proper
   diagonstics.
Please note:
   This routine is required for profiling and high resolution time
   measurement.
   It does not affect operation of the OS.
*/

OS_U32 OS_ConvertCycles2us(OS_U32 Cycles) {
  Cycles *= TIMER_PRESCALE;        /* eliminate effect of prescaler */
  return Cycles/(OS_FSYS/1000000);
}

/*********************************************************************
*
*       OS_Tick interrupt
*
**********************************************************************
*/

#pragma vector = 0x260
__interrupt void OS_ISR_Tick(void);  /* Avoid "has no prototype" warning */
__interrupt void OS_ISR_Tick(void) {
  OS_EnterNestableInterrupt();
  OS_HandleTick();
  OS_LeaveNestableInterrupt();
}

/*********************************************************************
*
*       Communication for embOSView V850 IA1 UART 1
*
**********************************************************************
*/

#if (OS_UART == 1)
  #define ASIMX0            ASIM10
  #define ASIMX1            ASIM11
  #define PRSCMX            PRSCM1
  #define PRSMX             PRSM1
  #define SRICX             SRIC1
  #define STICX             STIC1
#if ((OS_BAUDRATE==38400) && (OS_FSYS==40000000))
  /* 38400 at 40 MHz   */
  #define PRESCALE_MODE     0x80
  #define PRESCALE_COMPARE  0x10
#else
  #error Not currently supported ... Please define Prescale settings
#endif

void OS_COM_Init(void) {
  OS_DI();                /* Disable interrupts   */
  /* TXD,RXD pin configuration for UART1 or UART2 */
  PMC3 |= (0x03 << (2*OS_UART));
  PM3  |= (0x03 << (2*OS_UART));
  P3   &= ~(0x03 << (2*OS_UART));
  /* UART1 or UART 2 configuration */
  ASIMX0 = 0x08;          /* no parity, 8bit, stop1, baudrate gen */
  ASIMX1 = 0x00;          /* asynch. mode,1 frame send and receive*/
  PRSCMX = PRESCALE_COMPARE;
  PRSMX  = PRESCALE_MODE;
  ASIMX0 = 0x48;          /* receive enable                       */
  SRICX  = 0x07;          /* INTSR1  - UART1_RX, pri-lvl 7        */
  STICX  = 0x07;          /* INTST1  - UART1_TX, pri-lvl 7        */
  OS_RestoreI();          /* Restore Interrupts                   */
}

/*********************************************************************
*
*       OS_COM_Send1
*

Purpose: Transmitter Routine used by embOS for communication with
         embOSView.

Caution: NEVER call this routine from your application.

This routine depends on the serial interface used for communication
with embOSView. Therefore it is public here, so it can be modified
if you have to use an other UART for embOSView.
*/

void OS_COM_Send1(unsigned char c) {
  TXSL1 = c;
}

/*********************************************************************
*
*       OS_ISR_tx transmit interrupt handler
*/

#pragma vector = 0x370
__interrupt void OS_ISR_tx(void);  /* Avoid "has no prototype" warning */
__interrupt void OS_ISR_tx(void) {
  OS_EnterInterrupt();
  OS_EnterIntStack();
  OS_OnTx();
  OS_LeaveIntStack();
  OS_LeaveInterrupt();
}

/*********************************************************************
*
*       OS_ISR_rx receive interrupt handler
*/

static void OS_ISR_Rx_Handler(void) {
  volatile char Dummy;
  if (ASIS1 & 0x07) {           /* Check any reception error        */
    Dummy = RXBL1;              /* Reset error, discard Byte        */
  } else {
    OS_OnRx(RXBL1);             /* Process data                     */
  }
}

#pragma vector = 0x360
__interrupt void OS_ISR_rx(void);  /* Avoid "has no prototype" warning */
__interrupt void OS_ISR_rx(void) {
  OS_EnterNestableInterrupt();  /* We will enable interrupts        */
  OS_EnterIntStack();           /* We will use interrupt stack      */
  OS_ISR_Rx_Handler();          /* Call to handler is required !    */
  OS_LeaveIntStack();           /* Interrupt stack switching does   */
  OS_LeaveNestableInterrupt();  /* not allow local variables in ISR */
}

/********************************************************************/

#else        /* No supported UART selected; use dummy routines      */

/*********************************************************************
*
*       Dummy communication routines
*
**********************************************************************
*/

void OS_COM_Init(void) {}
void OS_COM_Send1(OS_U8 c) {
  OS_USEPARA(c);           /* Avoid warning       */
  OS_COM_ClearTxActive();  /* Release transmitter */
}
#endif

/****** End of file *************************************************/


