/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2012  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 3.86k                                            *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File:     RTOSInit_V850E_MS1.c

--------  END-OF-HEADER  --------------------------------------------*/


#include "RTOS.h"
#ifdef __ghs                    /* SFR file used with Green Hills   */
  #include <v850ems1.h>
#else
  #include <io703102.h>         /* SFR file delivered from IAR      */
#endif

/*********************************************************************
*
*       Configuration
*
**********************************************************************

  Define clock frequency for CPU: used to initilize embOS timer
  and baudrate for optional UART for embOSView

  Set UART and Baudrate for embOSView
  If you do not want (or can not due to hardware limitations) to
  dedicate a UART to embOSView, please define OS_UART to -1

*/

#ifndef   OS_FSYS
  #define OS_FSYS 40000000     /* default is 40 MHz              */
#endif
#ifndef   OS_UART
  #define OS_UART 1
#endif
#ifndef   OS_BAUDRATE
  #define OS_BAUDRATE 38400L
#endif
#define TIMER_PRESCALE  32     /* prescaler for timer frequency  */
#define SYS_CLK 1000           /* 1/SYS_CLK sec. is OS Timer     */

/*********************************************************************
*
*       OS_InitHW
*
**********************************************************************

Initialize the hardware required for the OS to run. This will work
on any target hardware, but may have to be tailored a little
(regarding the clock frequency). Of course the same holds true
if for some reason you choose to use an other timer.
*/

void OS_InitHW(void) {
  #define TIMER_RELOAD OS_FSYS/TIMER_PRESCALE/SYS_CLK
    /* start timer40   */
  TMC40   = 0x00;            /* stop timer */
  CM40    = TIMER_RELOAD;
  TMC40   = 0x80;            /* prescaler = /2/16 and start timer40 */
  CMIC40  = 0x07;
  OS_COM_Init();
}

/*********************************************************************
*
*       Idle Loop  (OS_Idle)
*
**********************************************************************

 Please note:
 This is basically the "core" of the idle loop.
 This core loop can be changed, but:
 The idle loop does not have a stack of its own, therefor no
 functionality should be implemented that relies on the stack
 to be preserved. However, a simple program loop can be programmed
 (like toggeling an output or incrementing a counter)
*/

void OS_Idle(void) {     /* Idle task: No task is ready to exec  */
  while (1);
}

/*********************************************************************
*
*       Get time [cycles]
*
**********************************************************************

    This routine is required for task-info via embOS Viewer or
    high resolution time measurement functions.
    It returns the system time in clock cycles of hardware timer.
*/

#define HW_TIMER TM40
#define HW_TIMER_INTERRUPT_PENDING() (CMIC40 & 0x80)

OS_U32 OS_GetTime_Cycles(void) {
  unsigned int t_cnt   = HW_TIMER;
  OS_U32 time    = OS_Time;
  if (HW_TIMER_INTERRUPT_PENDING()) {
    t_cnt = HW_TIMER;
    if (t_cnt == TIMER_RELOAD) {
      t_cnt =0;
    }
    time++;
  }
  return (OS_FSYS/TIMER_PRESCALE/SYS_CLK)*time  + t_cnt;
}

/*********************************************************************
*
*       OS_ConvertCycles2us
*
**********************************************************************

Purpose:
   Convert timer cycles into micro seconds.
   Used for Profiling or high resolution time measurement.
   If you use a clock frequency which is not a multiple of 1 MHz,
   you may have to modify this routine in order to get proper
   diagonstics.
Please note:
   This routine is required for profiling and high resolution time
   measurement.
   It does not affect operation of the OS.
*/

OS_U32 OS_ConvertCycles2us(OS_U32 Cycles) {
  Cycles *= TIMER_PRESCALE;        /* eliminate effect of prescaler */
  return Cycles/(OS_FSYS/1000000);
}

/*********************************************************************
*
*       OS_Tick interrupt
*
**********************************************************************
*/

#ifdef __ghs                    /* This declaration for Green Hills */
__interrupt void OS_ISR_Tick(void)
  #pragma intvect OS_ISR_Tick 0x280
#else                           /* This for IAR compiler            */
  #pragma vector = 0x280
__interrupt void OS_ISR_Tick(void);  /* Avoid "has no prototype" warning */
__interrupt void OS_ISR_Tick(void)
#endif
{
  OS_EnterNestableInterrupt();
  OS_HandleTick();
  OS_LeaveNestableInterrupt();
}

/*********************************************************************
*
*       Communication for embOSView V850E/MS1 UART 1
*
**********************************************************************
*/
#if defined (CPU_V850E_MS1) && (OS_UART==1)

void OS_COM_Init(void) {
  volatile char Dummy;
  #define BAUDDIVIDE ((OS_FSYS)/(OS_BAUDRATE*64))/*  uPClock / Baudrate*64   */
  ASIM10 = 0x08;                /* Disable Tx Rx, No Parity, 8 bit , 1 Stop  */
                                /* Baudrategenerator selected        */
  ASIM11 = 0;                   /* Expansion bit disabled            */
  CSIM1  = 0;                   /* Disable CSIO                      */
  BRGC1  = BAUDDIVIDE;          /* Setup Baudrate-Generator          */
  BPRM1  = 0x80;                /* Setup prescaler = 0, Enable Count */
  Dummy  = RXB1L;               /* Dummy read to clear Rx buffer     */
  SEIC1  |= 0x40;               /* Disable Rx error interrupt        */
  STIC1  &= ~0x40;              /* Enable Tx interupt                */
  SRIC1  &= ~0x40;              /* Enable Rx interrupt               */
  ASIM10 |= 0xC0;               /* Enable Tx Rx                      */
  PM2    |= (1<<5);             /* Make Tx pin output                */
  PM2    &= ~(1<<6);            /* Make Rx pin input                 */
  PMC2   |= (1<<5)|(1<<6);
}

/*********************************************************************
*
*       OS_COM_Send1
*

Purpose: Transmitter Routine used by embOS for communication with
         embOSView.

Caution: NEVER call this routine from your application.

This routine depends on the serial interface used for communication
with embOSView. Therefore it is public here, so it can be modified
if you have to use an other UART for embOSView.
*/

void OS_COM_Send1(unsigned char c) {
  TXS1L = c;
}

/*********************************************************************
*
*       OS_ISR_tx transmit interrupt handler
*/

#ifdef __ghs                    /* This declaration for Green Hills */
__interrupt void OS_ISR_tx(void)
  #pragma intvect OS_ISR_tx 0x370
#else                           /* This for IAR compiler            */
  #pragma vector = 0x370
__interrupt void OS_ISR_tx(void)
#endif
{
  OS_EnterInterrupt();
  OS_EnterIntStack();
  OS_OnTx();
  OS_LeaveIntStack();
  OS_LeaveInterrupt();
}

/*********************************************************************
*
*       OS_ISR_rx receive interrupt handler
*/
static void OS_ISR_Rx_Handler(void) {
  volatile char dummy;
  if (!(ASIS1 & 0x07)) {
    OS_OnRx(RXB1L);             /* Process data byte                */
  } else {
    dummy = RXB1L;              /* Reset error, discard byte        */
  }
}

#ifdef __ghs                    /* This declaration for Green Hills */
__interrupt void OS_ISR_rx(void)
  #pragma intvect OS_ISR_rx 0x360
#else                           /* This for IAR compiler            */
  #pragma vector = 0x360
__interrupt void OS_ISR_rx(void)
#endif
{
  OS_EnterNestableInterrupt();  /* We will enable interrupts        */
  OS_EnterIntStack();           /* We will use interrupt stack      */
  OS_ISR_Rx_Handler();          /* Call to handler is required !    */
  OS_LeaveIntStack();           /* Interrupt stack switching does   */
  OS_LeaveNestableInterrupt();  /* not allow local variables in ISR */
}

#else  /* No supported UART selected; use dummy routines            */

/*********************************************************************
*
*       Dummy communication routines
*
**********************************************************************
*/

void OS_COM_Init(void) {}
void OS_COM_Send1(OS_U8 c) {
  OS_USEPARA(c);           /* Avoid warning       */
  OS_COM_ClearTxActive();  /* Release transmitter */
}
#endif

/****** End of file *************************************************/

