/*********************************************************************
*               SEGGER MICROCONTROLLER GmbH & Co KG                  *
*       Solutions for real time microcontroller applications         *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2012  SEGGER Microcontroller GmbH & Co KG         *
*                                                                    *
*       www.segger.com     Support: support@segger.com               *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system for microcontrollers      *
*                                                                    *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: 3.86k                                            *
*                                                                    *
**********************************************************************

----------------------------------------------------------------------
File        : RTOSInit.c
Purpose     : Initializes and handles the hardware for embOS as far
              as required by embOS for V850 SB1.
              Feel free to modify this file acc. to your
              target system.
------------- END-OF-HEADER ------------------------------------------
*/


#include "RTOS.H"
#ifdef __ghs                    /* SFR file used with Green Hills   */
  #include <V850SB1.h>
#else
  #include <io70f3033ay.h>      /* SFR file delivered from IAR      */
#endif

/*********************************************************************
*
*       Configuration
*
**********************************************************************

  Define clock frequency for CPU: used to initilize embOS timer
  and baudrate for optional UART for embOSView

  Set UART and Baudrate for embOSView
  If you do not want (or can not due to hardware limitations) to
  dedicate a UART to embOSView, please define OS_UART to -1

*/

#ifndef   OS_FSYS
  #define OS_FSYS 20000000     /* default is 20 MHz */
#endif
#ifndef   OS_UART
  #define OS_UART 0
#endif
#ifndef   OS_BAUDRATE
  #define OS_BAUDRATE 38400L
#endif
#define TIMER_PRESCALE  2      /* prescaler for timer frequency  */
#define SYS_CLK 1000           /* 1/SYS_CLK sec. is OS Timer     */

/*********************************************************************
*
*       OS_InitHW
*
**********************************************************************

Initialize the hardware required for the OS to run. This will work
on any target hardware, but may have to be tailored a little
(regarding the clock frequency). Of course the same holds true
if for some reason you choose to use an other timer.
*/

void OS_InitHW(void) {
  #define TIMER_RELOAD OS_FSYS/TIMER_PRESCALE/SYS_CLK
  TMC1    = 0x00;               /* stop timer1                      */
  CRC1    = 0x00;               /* compare                          */
  CR10    = TIMER_RELOAD;
  PRM10   = 0x00;               /* prescaler = /2                   */
  PRM11   = 0x00;               /* prescaler = /2                   */
  TMIC10  = 0x07;               /* enable timer1 s         */
  TMC1    = 0x0c;               /* clear and restart on timer match */
  __DI();                       /* switch to full CPU speed         */
  PRCMD = 0;
  PCC  = 0;
  __asm("nop");
  __asm("nop");
  __EI();
  OS_COM_Init();
}

/*********************************************************************
*
*       Idle Loop  (OS_Idle)
*
**********************************************************************

 Please note:
 This is basically the "core" of the idle loop.
 This core loop can be changed, but:
 The idle loop does not have a stack of its own, therefore no
 functionality should be implemented that relies on the stack
 to be preserved. However, a simple program loop can be programmed
 (like toggeling an output or incrementing a counter)
*/

void OS_Idle(void) {     /* Idle loop: No task is ready to exec     */
  while (1);
}

/*********************************************************************
*
*       Get time [cycles]
*
**********************************************************************

    This routine is required for task-info via embOS Viewer or
    high resolution time measurement functions.
    It returns the system time in clock cycles of hardware timer.
*/

#define HW_TIMER TM1
#define HW_TIMER_INTERRUPT_PENDING() (TMIC10 & 0x80)

OS_U32 OS_GetTime_Cycles(void) {
  unsigned int t_cnt = HW_TIMER;
  OS_U32 time        = OS_Time;
  if (HW_TIMER_INTERRUPT_PENDING()) {
    t_cnt = HW_TIMER;
    if (t_cnt == TIMER_RELOAD) {
      t_cnt = 0;
    }
    time++;
  }
  return (OS_FSYS/TIMER_PRESCALE/SYS_CLK)*time + t_cnt;
}

/*********************************************************************
*
*       OS_ConvertCycles2us
*
**********************************************************************

Purpose:
   Convert timer cycles into micro seconds.
   Used for Profiling or high resolution time measurement.
   If you use a clock frequency which is not a multiple of 1 MHz,
   you may have to modify this routine in order to get proper
   diagonstics.
Please note:
   This routine is required for profiling and high resolution time
   measurement.
   It does not affect operation of the OS.
*/

OS_U32 OS_ConvertCycles2us(OS_U32 Cycles) {
  Cycles *= TIMER_PRESCALE;        /* eliminate effect of prescaler */
  return Cycles/(OS_FSYS/1000000);
}

/*********************************************************************
*
*       OS_Tick interrupt
*
**********************************************************************
*/

#ifdef __ghs                    /* This declaration for Green Hills */
__interrupt void OS_ISR_Tick(void)
  #pragma intvect OS_ISR_Tick 0x170
#else                           /* This for IAR compiler            */
  #pragma vector = 0x170
__interrupt void OS_ISR_Tick(void); /* Avoid "has no prototype" warning */
__interrupt void OS_ISR_Tick(void)
#endif
{
  OS_EnterNestableInterrupt();
  OS_HandleTick();
  OS_LeaveNestableInterrupt();
}

#if (OS_UART == 0)

/*********************************************************************
*
*       Communication for embOSView V850/SB1 UART 0
*
**********************************************************************
*/

/*  baud = OS_FSYS/[2^(BAUD_M) * BAUD_K] */
#if ((OS_BAUDRATE==19200) && (OS_FSYS==20000000))
  #define BAUD_M   3              // 19.200 at 20 MHz
  #define BAUD_K   130
#elif ((OS_BAUDRATE==38400) && (OS_FSYS==20000000))
  #define BAUD_M   3              /* 34800 at 20 MHz   */
  #define BAUD_K   65
#elif ((OS_BAUDRATE==38400) && (OS_FSYS==16000000))
  #define BAUD_M   3              /* 34800 at 20 MHz   */
  #define BAUD_K   52
#else
  #error Not currently supported ... Please define BAUD_M, BAUD_K
#endif

void OS_COM_Init(void) {
  volatile char Dummy;
  OS_DI();                /* Disable interrupts */
  CSIC1   |= 0x40;        /* Disable RX ISR     */
  STIC0   |= 0x40;        /* Disable TX ISR     */
  SERIC0  |= 0x40;        /* Disable ERR ISR    */
  PM1     = 0x2f;         /* TXD0 & RXD0        */
  P1      = 0x00;
  ASIM0   = 0x08;         /* 8N1, UART disabled, RxInt on error */
  BRGC0   = BAUD_K;
  BRGMC00 = BAUD_M;
  ASIM0   = 0xc8;         /* TX & RX,  8 Bit, 1 Stop, no parity */
  Dummy   = RXB0;         /* Dummy read to clear Rx buffer      */
  CSIC1   &= ~0x40;       /* enable RX ISR                      */
  STIC0   &= ~0x40;       /* enable TX ISR                      */
  OS_RestoreI();
}

/*********************************************************************
*
*       OS_COM_Send1
*

Purpose: Transmitter Routine used by embOS for communication with
         embOSView.

Caution: NEVER call this routine from your application.

This routine depends on the serial interface used for communication
with embOSView. Therefore it is public here, so it can be modified
if you have to use an other UART for embOSView.
*/

void OS_COM_Send1(unsigned char c) {
  TXS0 = c;
}

/*********************************************************************
*
*       OS_ISR_tx transmit interrupt handler
*/

#ifdef __ghs                    /* This declaration for Green Hills */
__interrupt void OS_ISR_tx(void)
  #pragma intvect OS_ISR_tx 0x220
#else                           /* This for IAR compiler            */
  #pragma vector = 0x220
__interrupt void OS_ISR_tx(void); /* Avoid "has no prototype" warning */
__interrupt void OS_ISR_tx(void)
#endif
{
  OS_EnterInterrupt();
  OS_EnterIntStack();
  OS_OnTx();
  OS_LeaveIntStack();
  OS_LeaveInterrupt();
}

/*********************************************************************
*
*       OS_ISR_rx receive interrupt handler
*/

static void OS_ISR_Rx_Handler(void) {
  volatile char Dummy;
  if (!ASIS0) {
    OS_OnRx(RXB0);              /* Process data byte                */
  } else {
    Dummy = RXB0;               /* Reset error, discard Byte        */
  }
}

#ifdef __ghs                    /* This declaration for Green Hills */
__interrupt void OS_ISR_rx(void)
  #pragma intvect OS_ISR_rx 0x210
#else                           /* This for IAR compiler            */
  #pragma vector = 0x210
__interrupt void OS_ISR_rx(void); /* Avoid "has no prototype" warning */
__interrupt void OS_ISR_rx(void)
#endif
{
  OS_EnterNestableInterrupt();  /* We will enable interrupts        */
  OS_EnterIntStack();           /* We will use interrupt stack      */
  OS_ISR_Rx_Handler();          /* Call to handler is required !    */
  OS_LeaveIntStack();           /* Interrupt stack switching does   */
  OS_LeaveNestableInterrupt();  /* not allow local variables in ISR */
}

#else  /* No supported UART selected; use dummy routines      */

/*********************************************************************
*
*       Dummy communication routines
*
**********************************************************************
*/

void OS_COM_Init(void) {}
void OS_COM_Send1(OS_U8 c) {
  OS_USEPARA(c);           /* Avoid warning       */
  OS_COM_ClearTxActive();  /* Release transmitter */
}
#endif

/****** End of file **************************************************/

